=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2009-12-17 10:01:25 +0000
+++ bzrlib/branch.py	2010-01-11 02:07:30 +0000
@@ -1449,7 +1449,7 @@
             format_string = transport.get_bytes("format")
             return klass._formats[format_string]
         except errors.NoSuchFile:
-            raise errors.NotBranchError(path=transport.base)
+            raise errors.NotBranchError(path=transport.base, bzrdir=a_bzrdir)
         except KeyError:
             raise errors.UnknownFormatError(format=format_string, kind='branch')
 
@@ -1804,7 +1804,7 @@
                               _repository=a_bzrdir.find_repository(),
                               ignore_fallbacks=ignore_fallbacks)
         except errors.NoSuchFile:
-            raise errors.NotBranchError(path=transport.base)
+            raise errors.NotBranchError(path=transport.base, bzrdir=a_bzrdir)
 
     def __init__(self):
         super(BranchFormatMetadir, self).__init__()

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2010-01-08 06:08:54 +0000
+++ bzrlib/bzrdir.py	2010-01-11 02:07:30 +0000
@@ -1034,19 +1034,6 @@
         except errors.NotBranchError:
             return False
 
-    def has_repository(self):
-        """Tell if this bzrdir contains a repository.
-
-        Note: if you're going to open the repository, you should just go ahead
-        and try, and not ask permission first.  (This method just opens the
-        repository and discards it, and that's somewhat expensive.)
-        """
-        try:
-            self.open_repository()
-            return True
-        except errors.NoRepositoryPresent:
-            return False
-
     def has_workingtree(self):
         """Tell if this bzrdir contains a working tree.
 
@@ -1478,12 +1465,7 @@
         return not isinstance(self._format, format.__class__)
 
     def open_branch(self, unsupported=False, ignore_fallbacks=False):
-        """See BzrDir.open_branch.
-        
-        Note: Test for repository on failure is not required here as
-        BzrDirPreSplitOut always has a branch; if there's no branch,
-        then there's no BzrDirPreSplitOut and no repository.
-        """
+        """See BzrDir.open_branch."""
         from bzrlib.branch import BzrBranchFormat4
         format = BzrBranchFormat4()
         self._check_supported(format, unsupported)
@@ -1660,16 +1642,9 @@
 
     def get_branch_reference(self):
         """See BzrDir.get_branch_reference()."""
-        try:
-            from bzrlib.branch import BranchFormat
-            format = BranchFormat.find_format(self)
-            return format.get_reference(self)
-        except errors.NotBranchError, e:
-            # provide indication to the user if they're actually
-            # opening a repository (see bug 440952)
-            if self.has_repository():
-                raise errors.NotBranchError(path=e.path, detail='location is a repository')
-            raise e
+        from bzrlib.branch import BranchFormat
+        format = BranchFormat.find_format(self)
+        return format.get_reference(self)
 
     def get_branch_transport(self, branch_format):
         """See BzrDir.get_branch_transport()."""
@@ -1715,15 +1690,6 @@
             pass
         return self.transport.clone('checkout')
 
-    def has_repository(self):
-        """See BzrDir.has_repository()."""
-        from bzrlib.repository import RepositoryFormat
-        try:
-            RepositoryFormat.find_format(self)
-            return True
-        except errors.NoRepositoryPresent:
-            return False
-
     def has_workingtree(self):
         """Tell if this bzrdir contains a working tree.
 
@@ -1777,16 +1743,9 @@
 
     def open_branch(self, unsupported=False, ignore_fallbacks=False):
         """See BzrDir.open_branch."""
-        try:
-            format = self.find_branch_format()
-            self._check_supported(format, unsupported)
-            return format.open(self, _found=True, ignore_fallbacks=ignore_fallbacks)
-        except errors.NotBranchError, e:
-            # provide indication to the user if they're actually
-            # opening a repository (see bug 440952)
-            if self.has_repository():
-                raise errors.NotBranchError(path=e.path, detail='location is a repository')
-            raise e
+        format = self.find_branch_format()
+        self._check_supported(format, unsupported)
+        return format.open(self, _found=True, ignore_fallbacks=ignore_fallbacks)
 
     def open_repository(self, unsupported=False):
         """See BzrDir.open_repository."""

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2010-01-08 06:08:54 +0000
+++ bzrlib/errors.py	2010-01-11 02:07:30 +0000
@@ -702,14 +702,31 @@
 # TODO: Probably this behavior of should be a common superclass
 class NotBranchError(PathError):
 
-    _fmt = 'Not a branch: "%(path)s"%(extra)s.'
+    _fmt = 'Not a branch: "%(path)s"%(detail)s.'
 
-    def __init__(self, path, detail=None):
+    def __init__(self, path, bzrdir=None):
        import bzrlib.urlutils as urlutils
        path = urlutils.unescape_for_display(path, 'ascii')
        # remember the detail in case of remote serialization
-       self.detail = detail
-       PathError.__init__(self, path=path, extra=self.detail)
+       self.detail = None
+       self.bzrdir = bzrdir
+       PathError.__init__(self, path=path)
+
+    def _format(self):
+        # XXX: Ideally self.detail would be a property, but Exceptions in
+        # Python 2.4 have to be old-style classes so properties don't work.
+        # Instead we override _format.
+        if self.detail is None:
+            if self.bzrdir is not None:
+                try:
+                    self.bzrdir.open_repository()
+                except NoRepositoryPresent:
+                    self.detail = ''
+                else:
+                    self.detail = ': location is a repository'
+            else:
+                self.detail = ''
+        return PathError._format(self)
 
 
 class NoSubmitBranch(PathError):

=== modified file 'bzrlib/tests/blackbox/test_shared_repository.py'
--- bzrlib/tests/blackbox/test_shared_repository.py	2009-10-15 16:56:24 +0000
+++ bzrlib/tests/blackbox/test_shared_repository.py	2010-01-11 02:07:30 +0000
@@ -126,6 +126,6 @@
         self.assertEqual(out, "")
         self.assertEqual(err, "")
         dir = BzrDir.open('a')
-        self.assertIs(dir.has_repository(), True)
+        dir.open_repository() # there is a repository there
         e = self.assertRaises(errors.NotBranchError, dir.open_branch)
-        self.assertEqual(e.detail, "location is a repository")
+        self.assertContainsRe(str(e), "location is a repository")

